<?php

namespace D3\XRechnungForOxid\Modules\Application\Model;

use OxidEsales\Eshop\Core\Registry;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\ClientException;

class Order extends Order_parent
{
    public function genXRechnung(string $sFilename, int $iSelLang = 0, bool $sendContent = false)
    {
        $blIsNewOrder = 0;
        // setting invoice number
        if (!$this->oxorder__oxbillnr->value) {
            $this->oxorder__oxbillnr->setValue($this->getNextBillNum());
            $blIsNewOrder = 1;
        }
        // setting invoice date
        if ($this->oxorder__oxbilldate->value == '0000-00-00') {
            $this->oxorder__oxbilldate->setValue(date('Y-m-d', mktime(0, 0, 0, date('m'), date('d'), date('Y'))));
            $blIsNewOrder = 1;
        }

        if ($blIsNewOrder) {
            $this->save();
        }

        // adding info data
        switch (\OxidEsales\Eshop\Core\Registry::getConfig()->getRequestParameter('exporttype')) {
            case ('xml'):
                return $this->getXRechnungXml($sFilename, $this->getId(), $iSelLang, $sendContent);
                break;
            default:
                return $this->getXRechnungXml($sFilename, $this->getId(), $iSelLang, $sendContent);
        }
    }

    /**
     * @param string $sFilename
     * @param string $sOrderId
     * @param int $iSelLang
     * @param bool $saveContent
     * @return true
     */
    protected function getXRechnungXml(string $sFilename, string $sOrderId, int $iSelLang,  bool $saveContent = false)
    {
        $oConfig = Registry::getConfig();

        // add payload
        $sPathDetails = $oConfig->getConfigParam('d3xrechnung_PATH_GENERATOR')."?";
        $query = [
            'orderid' => $sOrderId,
            'filename' => $sFilename,
            'iSelLang' => $iSelLang,
            'saveContent' => $saveContent,
            'combinexmlpdf' => false,
        ];

        // add accessdata
        $sUser = $oConfig->getConfigParam('d3xrechnung_HTACCESS_USER');
        $sPassword = $oConfig->getConfigParam('d3xrechnung_HTACCESS_PASSWORD');
        $auth = [$sUser, $sPassword]
;
        $client = new Client();
        try {
            $response = $client->request('GET', $sPathDetails,
                [
                    'auth' => $auth,
                    'query' => $query
                ]
            );
            $body = $response->getBody();
            return $body->getContents();
        }
        catch (ClientException $exception) {
            $logger = Registry::getLogger();
            $logger->error(Psr7\Message::toString($exception->getRequest()), [$exception]);
            $logger->error(Psr7\Message::toString($exception->getResponse()), [$exception]);
        }
        return true;
    }

    public function getBtFieldsXRechnung()
    {
        $sFields = $this->oxorder__d3btfields->getRawValue();

        $oConfig = Registry::getConfig();
        $aBtIndividualFields = (array)$oConfig->getConfigParam('d3xrechnung_BT_ORDERFIELDS_INDIVIDUAL');

        $aFields = unserialize($sFields);
        foreach ($aBtIndividualFields as $key => $Field)
        {
            if(!key_exists($Field, $aFields))
            {
                $aFields[$Field] = '';
            }
        }

        ksort($aFields, SORT_NATURAL);
        if(is_array($aFields))
        {
            return $aFields;

        }
        return array();
    }

    public function getExportFileTypes()
    {
        return array(
            'xml302' => 'XML (XRechnung 3.0.2)',
        );
    }
}
